@@ -1,3 +1,4 @@
+# please insert nothing before this line: -*- mode: cperl; cperl-indent-level: 4; cperl-continued-statement-offset: 4; indent-tabs-mode: nil -*-
 # Licensed to the Apache Software Foundation (ASF) under one or more
 # contributor license agreements.  See the NOTICE file distributed with
 # this work for additional information regarding copyright ownership.
@@ -73,13 +74,16 @@ my @c_scan_defines = (
     'CORE_PRIVATE',   #so we get all of apache
     'MP_SOURCE_SCAN', #so we can avoid some c-scan barfing
     '_NETINET_TCP_H', #c-scan chokes on netinet/tcp.h
+    '_BYTESWAP_H', #c-scan chokes on byteswap.h
+    '_BITS_BYTESWAP_H', #c-scan chokes on byteswap.h
+    'Expat_INCLUDED', #c-scan chokes on expath.h
  #   'APR_OPTIONAL_H', #c-scan chokes on apr_optional.h
     'apr_table_do_callback_fn_t=void', #c-scan chokes on function pointers
 );
 
 
 # some types c-scan failing to resolve
-push @c_scan_defines, map { "$_=void" } 
+push @c_scan_defines, map { "$_=void" }
     qw(PPADDR_t PerlExitListEntry modperl_tipool_vtbl_t);
 
 sub scan {
@@ -115,8 +119,16 @@ sub scan {
 
 sub include_dirs {
     my $self = shift;
-    ($self->config->apxs('-q' => 'INCLUDEDIR'),
-     $self->config->mp_include_dir);
+    my $inc = $self->config->apxs('-q' => 'INCLUDEDIR');
+    my @dirs = ($inc, $self->config->mp_include_dir);
+    my $aprinc = $self->config->apxs('-q' => 'APR_INCLUDEDIR');
+
+    unless ($aprinc eq $inc) {
+        # Add APR include directory if different to httpd includedir
+        push @dirs, $aprinc;
+    }
+
+    @dirs;
 }
 
 sub includes { shift->config->includes }
@@ -139,7 +151,8 @@ sub find_includes {
                                     apr_optional mod_include mod_cgi
                                     mod_proxy mod_ssl ssl_ apr_anylock
                                     apr_rmm ap_config mod_log_config
-                                    mod_perl modperl_ apreq);
+                                    mod_perl modperl_ apreq mod_cache
+                                    mod_serf mod_dav);
         $unwanted = qr|^$unwanted|;
         my $wanted = '';
 
@@ -238,10 +251,12 @@ my %defines_wanted = (
         mpmq       => [qw{AP_MPMQ_}],
         options    => [qw{OPT_}],
         override   => [qw{OR_ EXEC_ON_READ ACCESS_CONF RSRC_CONF}],
+        proxy      => [qw{PROXYREQ_}],
         platform   => [qw{CRLF CR LF}],
         remotehost => [qw{REMOTE_}],
         satisfy    => [qw{SATISFY_}],
         types      => [qw{DIR_MAGIC_TYPE}],
+        auth       => [qw{AUTHN_ AUTHZ AP_AUTH_ AUTH_ AUTHZ_}],
     },
     'APR::Const' => {
         common    => [qw{APR_SUCCESS}],
@@ -274,12 +289,15 @@ while (my ($class, $groups) = each %defines_wanted) {
 }
 
 my %enums_wanted = (
-    'Apache2::Const' => { map { $_, 1 } qw(cmd_how input_mode filter_type conn_keepalive) },
+    'Apache2::Const' => { map { $_, 1 } qw(cmd_how input_mode filter_type conn_keepalive authn_status authz_status) },
     'APR::Const' => { map { $_, 1 } qw(apr_shutdown_how apr_read_type apr_lockmech) },
 );
 
 my $defines_unwanted = join '|', qw{
 HTTP_VERSION APR_EOL_STR APLOG_MARK APLOG_NOERRNO APR_SO_TIMEOUT
+APR_HOOK_PROBES_ENABLED APR_HOOK_INT_DCL_UD
+APLOG_MAX_LOGLEVEL
+APR_BEGIN_DECLS APR_END_DECLS
 };
 
 sub get_constants {
@@ -314,12 +332,12 @@ sub get_constants {
 
 sub handle_constant {
     my ($self, $constants) = @_;
-    my $keys = keys %defines_wanted_re; #XXX broken bleedperl ?
+    my $keys = keys %defines_wanted_re;
 
     return if /^($defines_unwanted)/o;
 
     while (my ($class, $groups) = each %defines_wanted_re) {
-        my $keys = keys %$groups; #XXX broken bleedperl ?
+        my $keys = keys %$groups;
 
         while (my ($group, $re) = each %$groups) {
             next unless /^($re)/;
@@ -398,6 +416,8 @@ sub get_functions {
     my $c = $self->{c};
 
     my $fdecls = $c->get($key);
+    my $inlines = $c->get('parsed_inlines');
+    push @{$fdecls}, @{$inlines};
 
     my %seen;
     my $wanted = $self->wanted_functions;
@@ -520,7 +540,16 @@ sub write_pm {
 
     my ($subdir) = (split '::', $name)[0];
 
-    my $tdir = 'xs/tables/current';
+    my $tdir = '';
+    my $build = Apache2::Build->new(init => 1);
+    my $httpd_version = $build->httpd_version;
+    if ($httpd_version lt '2.4.0') {
+        $tdir='xs/tables/current';
+    }
+    else {
+        $tdir='xs/tables/current24';
+    }
+
     if (-d "$tdir/$subdir") {
         $file = "$tdir/$subdir/$file";
     }
